#!/usr/bin/env python

"""
usage:
  flask-restful-crud-template.py [-h] [-v] database_file

synopsis:
  Run flask-rest app with SQLObject database. Implements REST CRUD
  access to a set of tasks (name, description).

positional arguments:
  database_file  The name of the database file

optional arguments:
  -h, --help     show this help message and exit
  -v, --verbose  Print messages during actions.
"""

# Try to make our app run under both Python 2 and Python 3.
from __future__ import print_function
#import sys
import os
import argparse
from flask import Flask, request
from flask_restful import abort, Api, Resource
import sqlobject

app = Flask(__name__)
api = Api(app)

#
# The following can be used for configuration.
##app.config.update(
##    DEBUG=True,
##    #SERVER_NAME='localhost:8080',
##    #SERVER_NAME='localhost:5000',
##    SERVER_NAME='localhost:8001',
##)

# Define the host and post.
SERVER_HOST = 'localhost'
SERVER_PORT = 8001

CMD_LINE_OPTIONS = None


def dbg_msg(msg):
    """Print a message if verbose is on."""
    if CMD_LINE_OPTIONS.verbose:
        print(msg)


#
# Define the objects to be stored in the SQLObject database.
class TaskDef(sqlobject.SQLObject):
    task_name = sqlobject.StringCol()
    description = sqlobject.StringCol()


def open_db(db_filename):
    """Open the database.  If it does not exist, create it."""
    create = False
    db_filename = os.path.abspath(db_filename)
    if not os.path.exists(db_filename):
        create = True
    connection_string = 'sqlite:' + db_filename
    connection = sqlobject.connectionForURI(connection_string)
    sqlobject.sqlhub.processConnection = connection
    dbg_msg('Openned database file: {}'.format(
        CMD_LINE_OPTIONS.database_file))
    if create:
        TaskDef.createTable()
        dbg_msg('Created table')


def get_task_by_name(task_name):
    """Retrieve a task from the DB.  Abort if it does not exist."""
    results = TaskDef.select(TaskDef.q.task_name == task_name)
    if results.count() < 1:
        abort(404, message="Task {} doesn't exist".format(task_name))
    return results[0]


class TaskList(Resource):
    """Handle operations on the list of tasks.

    Enable a GET command to get a list of tasks or a single task.
    Enable a POST command to create and add a new task.
    Enable a PUT command to update a task.
    """
    def __init__(self, *args, **kwargs):
        super(TaskList, self).__init__(*args, **kwargs)

    def get(self, task_name=None):
        """Return a collection of existing tasks or a single task."""
        #
        # begin fill-in
        # ---------------------------------------
        if task_name is None:
            dbg_msg('Retrieving list of tasks')
            collection = {}
            results = TaskDef.select()
            for task in results:
                collection[task.task_name] = task.description
            content = {
                'action': 'get-list',
                'collection': collection}
            return content
        else:
            dbg_msg('Retrieving task: {}'.format(task_name))
            task = get_task_by_name(task_name)
            content = {
                'action': 'get-item',
                'task_name': task_name,
                'description': task.description,
            }
            return content
        # ---------------------------------------
        # end fill-in
        #

    def delete(self, task_name):
        """Delete a single task."""
        #
        # begin fill-in
        # ---------------------------------------
        dbg_msg('Deleting task: {}'.format(task_name))
        task = get_task_by_name(task_name)
        TaskDef.delete(task.id)
        task_json = {
            'action': 'delete',
            'task_name': task_name,
            'description': task.description}
        return task_json
        #return '', 204
        # ---------------------------------------
        # end fill-in
        #

    def post(self):
        """Create and save a new task."""
        #
        # begin fill-in
        # ---------------------------------------
        task_name = request.form['task_name']
        dbg_msg('Creating task: {}'.format(task_name))
        description = request.form['description']
        TaskDef(
            task_name=task_name,
            description=description)
        task_json = {
            'action': 'create',
            'task_name': task_name,
            'description': description}
        return task_json
        # ---------------------------------------
        # end fill-in
        #

    def put(self):
        """Create and save a new task."""
        #
        # begin fill-in
        # ---------------------------------------
        task_name = request.form['task_name']
        dbg_msg('Updating task: {}'.format(task_name))
        description = request.form['description']
        task = get_task_by_name(task_name)
        task.description = description
        task_json = {
            'action': 'update',
            'task_name': task_name,
            'description': description}
        return task_json
        # ---------------------------------------
        # end fill-in
        #


#
# Set up the api resource routing.
#
api.add_resource(
    TaskList,
    # Get a list of tasks.
    '/tasks/gettask',
    # Get one task by name.
    '/tasks/gettask/<task_name>',
    # Add a task.  Form data: task_name, description.
    '/tasks/addtask',
    # Update a task.  Form data: task_name, description.
    '/tasks/updatetask',
    # Delete one task.`
    '/tasks/delete/<string:task_name>',
)


def main():
    global CMD_LINE_OPTIONS
    description = """\
Run flask-rest app with SQLObject database.
Implements REST CRUD access to a set of tasks (name, description).
"""
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument(
        "database_file",
        help="The name of the database file"
    )
    parser.add_argument(
        "-v", "--verbose",
        action="store_true",
        help="Print messages during actions."
    )
    CMD_LINE_OPTIONS = parser.parse_args()
    db_filename = CMD_LINE_OPTIONS.database_file
    open_db(db_filename)
    app.run(debug=True, host=SERVER_HOST, port=SERVER_PORT)


if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()
